package com.jspgou.cms.web;

import java.lang.reflect.Method;
import java.sql.Timestamp;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import org.springframework.web.util.UrlPathHelper;

import com.jspgou.cms.web.SignValidate;
import com.jspgou.cms.api.ApiResponse;
import com.jspgou.cms.api.ApiValidate;
import com.jspgou.cms.api.Constants;
import com.jspgou.cms.api.ResponseCode;
import com.jspgou.cms.entity.ApiAccount;
import com.jspgou.cms.entity.ApiUserLogin;
import com.jspgou.cms.entity.ShopAdmin;
import com.jspgou.cms.manager.ApiAccountMng;
import com.jspgou.cms.manager.ApiUserLoginMng;
import com.jspgou.common.util.AES128Util;
import com.jspgou.common.web.CookieUtils;
import com.jspgou.common.web.ResponseUtils;
import com.jspgou.common.web.session.SessionProvider;
import com.jspgou.core.entity.Website;
import com.jspgou.core.manager.WebsiteMng;
import com.jspgou.core.web.WebErrors;

/**
 * 后台API拦截器
 *判断用户权限信息
 */
public class AdminApiInterceptor extends HandlerInterceptorAdapter {
	private static final Logger log = Logger
			.getLogger(AdminApiInterceptor.class);
	public static final String SITE_PARAM = "_site_id_param";
	public static final String SITE_COOKIE = "_site_id_cookie";
	public static final String PERMISSION_MODEL = "_permission_key";

	@Override
	public boolean preHandle(HttpServletRequest request,
			HttpServletResponse response, Object handler) throws Exception {
		String body="\"\"";
		int code= ResponseCode.API_INIT ;
		String message = "";
		try {
			// 正常状态
			WebErrors errors=WebErrors.create(request);
			String uri = getURI(request);
			//验证appId是否有效
			ApiAccount apiAccount=getApiAccount(request);
			if(apiAccount!=null&&!apiAccount.getDisabled()){
				CmsThreadVariable.setApiAccount(apiAccount);
				// 不在验证的范围内
				if (exclude(excludeUrls,uri)) {
					return true;
				}
				//获取用户
				ShopAdmin user=getUser(apiAccount,request);
				//获取用户状态
				Short userStatus=getStatus(apiAccount,request);
				CmsUtils.setAdmin(request, user);
				// User加入线程变量
				CmsThreadVariable.setAdminUser(user);
				if(null != user){
					if(user.getAdmin() != null &&  user.getAdmin() !=null && user.getAdmin().getDisabled()){
						code=ResponseCode.API_CODE_ADMIN_ENABLE;
						message = Constants.API_MESSAGE_ADMIN_ENABLE;
					}else{
						// Site加入线程变量及request中
						CmsThreadVariable.setSite(user.getWebsite());
						CmsUtils.setWebsite(request, user.getWebsite());
						//判断用户活跃状态 1-已登录   2-未登录  3-登录超时   
						if(userStatus.equals(ApiUserLogin.USER_STATUS_LOGOVERTIME)){
							code=ResponseCode.API_CODE_USER_OVER_TIME;
							message = Constants.API_MESSAGE_USER_OVER_TIME;
						}else if(userStatus.equals(ApiUserLogin.USER_STATUS_LOGOUT)){
							code=ResponseCode.API_CODE_USER_NOT_LOGIN;
							message = Constants.API_MESSAGE_USER_NOT_LOGIN;
						}else{
							//校验当前用户是否拥有超级管理员权限且非只读管理员
							if(!user.isSuper()){
								//用户是否拥有本API调用的权限
								if(!hasUrlPerm(user.getWebsite(), user, uri)){
									code=ResponseCode.API_CODE_USER_NOT_HAS_PERM;
									message = Constants.API_MESSAGE_USER_NOT_HAS_PERM;
								}
							}
							if(code == ResponseCode.API_INIT){
								if (exclude(excludeUeditorUrls,uri)) {
									return true;
								}
								HandlerMethod handlerMethod = (HandlerMethod) handler;
					            Method method = handlerMethod.getMethod();
					            SignValidate annotation = method.getAnnotation(SignValidate.class);
					            //判断api接口是否加入校验签名注解，开发阶段可取消 
									if(annotation != null && annotation.need()){
										//拦截只读用户
										if(user.getAdmin().getViewonlyAdmin()){
											code=ResponseCode.API_CODE_USER_NOT_HAS_PERM;
											message = Constants.API_MESSAGE_USER_NOT_HAS_PERM;
										}else{
											Object[]result=validateSign(request,errors);
											if(!"0".equals(result[0])){
												code = Integer.parseInt(String.valueOf(result[0]));
												message = String.valueOf(result[1]);
											}
										}
								   }
							}
						}
					}
				}else{
					//用户不存在，即未登录
					code=ResponseCode.API_CODE_USER_NOT_LOGIN;
					message = Constants.API_MESSAGE_USER_NOT_LOGIN;
				}
			}else{
				code=ResponseCode.API_CODE_APP_PARAM_ERROR;
				message = Constants.API_MESSAGE_APP_PARAM_ERROR;
			}
		} catch (Exception e) {
			// TODO: handle exception
			e.printStackTrace();
			code=ResponseCode.API_CODE_CALL_FAIL;
			message = Constants.API_MESSAGE_FAIL;
		}
		if(code != ResponseCode.API_INIT){
			ApiResponse apiResponse=new ApiResponse(body, message,code);
			ResponseUtils.renderApiJson(response, request, apiResponse);
			return false;
		}
		//刷新用户活跃时间
		refreshUserLoginActive();
		return true;
	}
	
	@Override
	public void postHandle(HttpServletRequest request,
			HttpServletResponse response, Object handler, ModelAndView mav)
			throws Exception {

	}

	/**
	 * 按参数、cookie、域名、默认。
	 * 
	 * @param request
	 * @return 不会返回null，如果站点不存在，则抛出异常。
	 */
	@SuppressWarnings("unused")
	private Website getSite(HttpServletRequest request,
			HttpServletResponse response) {
		Website site = getByParams(request, response);
		if (site == null) {
			site = getByCookie(request);
		}
		if (site == null) {
			site = getByDomain(request);
		}
		if (site == null) {
			site = getByDefault();
		}
		if (site == null) {
			throw new RuntimeException("cannot get site!");
		} else {
			return site;
		}
	}

	private Website getByParams(HttpServletRequest request,
			HttpServletResponse response) {
		String p = request.getParameter(SITE_PARAM);
		if (!StringUtils.isBlank(p)) {
			try {
				Long siteId = Long.parseLong(p);
				Website site = websiteMng.findById(siteId);
				if (site != null) {
					// 若使用参数选择站点，则应该把站点保存至cookie中才好。
					CookieUtils.addCookie(request, response, SITE_COOKIE, site
							.getId().toString(), null,null);
					return site;
				}
			} catch (NumberFormatException e) {
				log.warn("param site id format exception", e);
			}
		}
		return null;
	}

	private Website getByCookie(HttpServletRequest request) {
		Cookie cookie = CookieUtils.getCookie(request, SITE_COOKIE);
		if (cookie != null) {
			String v = cookie.getValue();
			if (!StringUtils.isBlank(v)) {
				try {
					Long siteId = Long.parseLong(v);
					return websiteMng.findById(siteId);
				} catch (NumberFormatException e) {
					log.warn("cookie site id format exception", e);
				}
			}
		}
		return null;
	}

	private Website getByDomain(HttpServletRequest request) {
		String domain = request.getServerName();
		if (!StringUtils.isBlank(domain)) {
			return websiteMng.findByDomain(domain, true);
		}
		return null;
	}

	private Website getByDefault() {
		List<Website> list = websiteMng.getListFromCache();
		if (list.size() > 0) {
			return list.get(0);
		} else {
			return null;
		}
	}

	private boolean exclude(String[] urs,String uri) {
		if (urs != null) {
			for (String exc : urs) {
				if (exc.equals(uri)) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * 获得第三个路径分隔符的位置
	 * 
	 * @param request
	 * @throws IllegalStateException
	 *             访问路径错误，没有三(四)个'/'
	 */
	private static String getURI(HttpServletRequest request)
			throws IllegalStateException {
		UrlPathHelper helper = new UrlPathHelper();
		String uri = helper.getOriginatingRequestUri(request);
		String ctxPath = helper.getOriginatingContextPath(request);
		int start = 0, i = 0, count = 2;
		if (!StringUtils.isBlank(ctxPath)) {
			count++;
		}
		while (i < count && start != -1) {
			start = uri.indexOf('/', start + 1);
			i++;
		}
		if (start <= 0) {
			throw new IllegalStateException(
					"admin access path not like '/jeeadmin/jeecms/...' pattern: "
							+ uri);
		}
		return uri.substring(start);
	}
	
	private ApiAccount getApiAccount(HttpServletRequest request){
		String appId=request.getParameter(this.appId);
		ApiAccount apiAccount = null;
		if(StringUtils.isNotBlank(sessionKey)&&StringUtils.isNotBlank(appId)){
			apiAccount=apiAccountMng.findByAppId(appId);
		}
		return apiAccount;
	}
	
	private ShopAdmin getUser(ApiAccount apiAccount,
			HttpServletRequest request) throws Exception{
 		ShopAdmin user=null;
		String sessionKey=request.getParameter(this.sessionKey);
		String aesKey=apiAccount.getAesKey();
		user=apiUserLoginMng.findUser(sessionKey, aesKey, apiAccount.getIvKey());
		return user;
	}
	
	private Short getStatus(ApiAccount apiAccount,HttpServletRequest request){
		String sessionKey=request.getParameter(this.sessionKey);
		Short loginStatus=ApiUserLogin.USER_STATUS_LOGOUT;
		if(apiAccount!=null){
			String decryptSessionKey="";
			String aesKey=apiAccount.getAesKey();
			try {
				decryptSessionKey=AES128Util.decrypt(sessionKey, aesKey,apiAccount.getIvKey());
			} catch (Exception e) {
				//e.printStackTrace();
			}
			if(StringUtils.isNotBlank(decryptSessionKey)){
				loginStatus=apiUserLoginMng.getUserStatus(decryptSessionKey);
			}
		}
		return loginStatus;
	}
	
	private Object[] validateSign(HttpServletRequest request,WebErrors errors){
		String sign=request.getParameter(this.sign);
		String appId=request.getParameter(this.appId);
		ApiAccount apiAccount=apiAccountMng.findByAppId(appId);
		errors=ApiValidate.validateApiAccount(request, errors, apiAccount);
		int code= 0 ;
		String message = "";
		Object[]result=new Object[2];
		if(errors.hasErrors()){
			code=ResponseCode.API_CODE_APP_PARAM_ERROR;
			message =  Constants.API_MESSAGE_APP_PARAM_ERROR;
		}else{
			//验证签名
			errors=ApiValidate.validateSign(request, errors,apiAccount, sign);
			//Account可能获取不到，需要再次判断
			if(errors.hasErrors()){
				code=ResponseCode.API_CODE_SIGN_ERROR;
				message = Constants.API_MESSAGE_SIGN_VALIDATE_ERROR;
			}
		}
		result[0]=code;
		result[1]=message;
		return result;
	}
	
	/**
	 * 刷新用户的活跃时间,即重置用户登录有效期计算的起始时间
	 */
	private void refreshUserLoginActive(){
		ApiUserLogin apiUserLogin =  CmsThreadVariable.getApiUserLogin();
		if(apiUserLogin != null){
			apiUserLogin.setActiveTime( new Timestamp(System.currentTimeMillis()));
			apiUserLoginMng.update(apiUserLogin);
		}
	}
	
	
	@SuppressWarnings("unused")
	private void userActive(HttpServletRequest request){
		String sessionKey=request.getParameter(this.sessionKey);
		ApiAccount apiAccount = getApiAccount(request);
		Short status=getStatus(apiAccount,request);
		if(apiAccount!=null){
			String decryptSessionKey="";
			String aesKey=apiAccount.getAesKey();
			try {
				decryptSessionKey=AES128Util.decrypt(sessionKey, aesKey,apiAccount.getIvKey());
			} catch (Exception e) {
				//e.printStackTrace();
			}
			if(StringUtils.isNotBlank(decryptSessionKey)){
				apiUserLoginMng.userActive(decryptSessionKey);
			}
		}
		if(apiAccount!=null&&status.equals(ApiUserLogin.USER_STATUS_LOGIN)){
			apiUserLoginMng.userActive(sessionKey);
		}
	}
	
	private boolean hasUrlPerm(Website site,ShopAdmin user,String url){
		Set<String> perms=getUserPermission(site, user);
		if (perms == null) {
			return true;
		}else{
			Iterator<String>it =perms.iterator();
			while(it.hasNext()){
				String perm =it.next();
				if (perm.equals("/*")||perm.endsWith(url)) {
					return true;
				}
			}
		}
		return false;
	}
	
	private Set<String>getUserPermission(Website site,ShopAdmin user){
		Set<String> perms = user.getPerms();
		Set<String> userPermission=new HashSet<String>();
		if(perms!=null){
			for(String perm:perms){ 
				if(perm.contains(":")){
					perm=perm.replace(":", "/").replace("*", "");
				}
				userPermission.add(perm);
			}
		}
		return userPermission;
	}

	@Autowired
	private WebsiteMng websiteMng;
	@Autowired
	private SessionProvider session;

	private String[] excludeUrls;
	private String[] excludeUeditorUrls;
	private String sessionKey="sessionKey";
	private String appId="appId";
	private String sign="sign";
	@Autowired
	private ApiAccountMng apiAccountMng;
	@Autowired
	private ApiUserLoginMng apiUserLoginMng;

	public void setExcludeUrls(String[] excludeUrls) {
		this.excludeUrls = excludeUrls;
	}

	public void setExcludeUeditorUrls(String[] excludeUeditorUrls) {
		this.excludeUeditorUrls = excludeUeditorUrls;
	}
	
	
}