package com.jspgou.cms.api.admin;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import com.jspgou.common.web.RequestUtils;
import com.jspgou.common.web.ResponseUtils;
import com.jspgou.cms.manager.ApiUserLoginMng;
import com.jspgou.common.web.LoginUtils;
import com.jspgou.common.web.session.SessionProvider;
import com.jspgou.common.security.encoder.PwdEncoder;
import com.jspgou.common.util.AES128Util;
import com.jspgou.cms.entity.ShopAdmin;
import com.jspgou.cms.api.ApiValidate;
import com.jspgou.cms.entity.ApiAccount;
import com.jspgou.cms.manager.ApiRecordMng;
import com.jspgou.cms.manager.ShopAdminMng;
import com.jspgou.cms.web.CmsThreadVariable;
import com.jspgou.cms.api.ApiResponse;
import com.jspgou.cms.api.Constants;
import com.jspgou.cms.api.ResponseCode;
import com.jspgou.core.web.WebErrors;

@Controller
public class LoginController {
	
	@Autowired
	private ApiRecordMng apiRecordMng;
	@Autowired
	private ShopAdminMng shopAdminMng;
	@Autowired
	private PwdEncoder  pwdEncodee;
	@Autowired
	private SessionProvider session;
	@Autowired
	private ApiUserLoginMng apiUserLoginMng;
	
	/**
	 * 用户登录
	 * @param userName用户名
	 * @param passWord 密码
	 * @param appId appID
	 * @param nonce_str  随机字符
	 * @param sign  签名
	 * @param request
	 * @param response
	 */
 	@RequestMapping(value="/user/login",method=RequestMethod.POST)
	public void login(String userName,String passWord,String appId,String nonce_str,String sign,
			HttpServletRequest request,HttpServletResponse response){
 		String body="\"\"";
 		String message="\"\"";
 		int code=ResponseCode.API_INIT;
 		try {
			WebErrors errors=WebErrors.create(request);
			//验证公共非空参数
			errors=ApiValidate.validateRequiredParams(errors, appId,
					nonce_str,sign,userName,passWord);
			if(errors.hasErrors()){
				code=ResponseCode.API_CODE_PARAM_ERROR;
				message = Constants.API_MESSAGE_PARAM_ERROR;
			}else{
				//通过线程获取当前请求的app所使用的账户
				ApiAccount apiAccount = CmsThreadVariable.getApiAccount();
				//验证签名
				errors=ApiValidate.validateSign(request, errors,apiAccount, sign);
				if(errors.hasErrors()){
					code=ResponseCode.API_CODE_SIGN_ERROR;
					message = Constants.API_MESSAGE_SIGN_VALIDATE_ERROR;
				}else{
					ShopAdmin user=shopAdminMng.getByUsername(userName);
					if(null != user){
						String encryptPass = AES128Util.decrypt(passWord, apiAccount.getAesKey(),apiAccount.getIvKey());
						//校验密码
						if(pwdEncodee.isPasswordValid(user.getAdmin().getUser().getPassword(), encryptPass)){
							//获取sessionId
							session.logout(request, response);  //防止登陆之前登陆一个错误的账户，占用了session，此处重置再登陆
							String sessionKey=session.getSessionId(request, response);
							//更新或保存用户登录记录
							apiUserLoginMng.userLogin(userName, sessionKey);
							//调用shrio登录，使前台共享后台登录用户
							LoginUtils.loginShiro(request, response, userName);
							//加密返回
							//新增或更新api请求记录
							apiRecordMng.callApiRecord(RequestUtils.getIpAddr(request),
									appId, "/user/login",sign);
							code=ResponseCode.API_CODE_CALL_SUCCESS;
							message=Constants.API_MESSAGE_SUCCESS;
							JSONObject json =  new JSONObject();
							json.put("sessionKey", AES128Util.encrypt(sessionKey, apiAccount.getAesKey(),apiAccount.getIvKey()));
							if(user.isSuper()){
								json.put("permission", "*");
							}else{
								json.put("permission", StringUtils.join(user.getPerms().toArray(), ","));
							}
							body=json.toString();
						}else{
							//密码错误
							code=ResponseCode.API_CODE_USERNAME_OR_PASSWORD_ERROR;
							message=Constants.API_MESSAGE_USERNAME_OR_PASSWORD_ERROR;
						}
					}else{
						//用户不存在，即未登录
						code=ResponseCode.API_CODE_USERNAME_OR_PASSWORD_ERROR;
						message = Constants.API_MESSAGE_USERNAME_OR_PASSWORD_ERROR;
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			code=ResponseCode.API_CODE_CALL_FAIL;
			message=Constants.API_MESSAGE_FAIL;
		}
		ApiResponse apiResponse = new ApiResponse(body, message, code);
 		ResponseUtils.renderApiJson(response, request, apiResponse);
	}
	
 
 	/**
 	 * 退出登录
 	 * @param sessionKey    sessionKey
 	 * @param appId    appiId
 	 * @param nonce_str 随机字符
 	 * @param sign 签名
 	 * @param request
 	 * @param response
 	 */
 	@RequestMapping(value="/user/loginOut")
 	public void loginOut(String sessionKey,String appId,String nonce_str,String sign,HttpServletRequest request,HttpServletResponse response){
 		String body="\"\"";
 		String message="\"\"";
 		int code=ResponseCode.API_INIT;
 		try {
			WebErrors errors=WebErrors.create(request);
			//验证公共非空参数
			errors=ApiValidate.validateRequiredParams(errors, appId,
					nonce_str,sign,sessionKey);
			if(errors.hasErrors()){
				code=ResponseCode.API_CODE_PARAM_ERROR;
				message = Constants.API_MESSAGE_PARAM_ERROR;
			}else{
				//通过线程获取当前请求的app所使用的账户
				ApiAccount apiAccount = CmsThreadVariable.getApiAccount();
				//验证签名
				errors=ApiValidate.validateSign(request, errors,apiAccount, sign);
				if(errors.hasErrors()){
					code=ResponseCode.API_CODE_SIGN_ERROR;
					message = Constants.API_MESSAGE_SIGN_VALIDATE_ERROR;
				}else{
					ShopAdmin user=apiUserLoginMng.findUser(sessionKey, apiAccount.getAesKey(), apiAccount.getIvKey());
					if(null != user){
							apiUserLoginMng.userLogout(user.getUsername());
							LoginUtils.logout();
					}
					code=ResponseCode.API_CODE_CALL_SUCCESS;
					message=Constants.API_MESSAGE_SUCCESS;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			code=ResponseCode.API_CODE_CALL_FAIL;
			message=Constants.API_MESSAGE_FAIL;
		}
		ApiResponse apiResponse = new ApiResponse(body, message, code);
 		ResponseUtils.renderApiJson(response, request, apiResponse);
 	}

 	/**
 	 * 通过sessionKey拉取用户相应信息
 	 * @param sessionKey
 	 * @param request
 	 * @param response
 	 */
 	@RequestMapping(value="/user/getLoginUser")
 	public void getLoginUser(String sessionKey,HttpServletRequest request,HttpServletResponse response){
 		String body="\"\"";
 		int code=ResponseCode.API_CODE_CALL_SUCCESS;
 		String message=Constants.API_MESSAGE_SUCCESS;
 		try {
			WebErrors errors=WebErrors.create(request);
			//验证公共非空参数
			errors=ApiValidate.validateRequiredParams(errors, sessionKey);
			if(errors.hasErrors()){
				code=ResponseCode.API_CODE_PARAM_ERROR;
				message = Constants.API_MESSAGE_PARAM_ERROR;
			}else{
				//通过线程获取当前请求的app所使用的账户
				ApiAccount apiAccount = CmsThreadVariable.getApiAccount();
				ShopAdmin user=apiUserLoginMng.findUser(sessionKey, apiAccount.getAesKey(), apiAccount.getIvKey());;
				if(user != null){
					JSONObject json =  new JSONObject();
					if(user.isSuper()){
						json.put("permission", "*");
					}else{
						json.put("permission", StringUtils.join(user.getPerms().toArray(), ","));
					}
					body=json.toString();
				}else{
					code=ResponseCode.API_CODE_USER_NOT_LOGIN;
					message = Constants.API_MESSAGE_USER_NOT_LOGIN;
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			code=ResponseCode.API_CODE_CALL_FAIL;
			message=Constants.API_MESSAGE_FAIL;
		}
		ApiResponse apiResponse = new ApiResponse(body, message, code);
 		ResponseUtils.renderApiJson(response, request, apiResponse);
 	}
 	
}
